%% ------------------------------- METADATA -------------------------------
% Title: Convex LPV synthesis of estimators and feedforwards using integral
% quadratic constraints
% Authors: Raghu Venkataraman and Peter Seiler
% Affiliation: Department of Aerospace Engineering and Mechanics,
% University of Minnesota
%--------------------------------------------------------------------------

%% ------------------------------ DESCRIPTION -----------------------------
% ----- Feedforward synthesis example for spring-mass-damper system -------
% There are 2 springs, 2 masses, and 2 dampers. The spring connecting the
% two masses is nonlinear, wherein the nonlinearity is constrained to lie
% within a known sector. The objective is for the position of the second
% mass to track a reference command, through a force input applied at the
% first mass.
%--------------------------------------------------------------------------

%% Setup
clearvars, close all, clc
warning('off','Control:ltiobject:MultipleChannelMatch');
warning('off','Control:ltiobject:MultipleGroupMatch');

%% Definition of nominal and uncertain parts
% Define parameter-varying mode
PVMode = 'LPV';

% Definition of basic plant describing sping-mass-damper system
m1 = 1; m2 = 0.5; k1 = 1; k2 = 1; c2 = 2; % known system parameters
switch PVMode
    case 'LTI'
        c1 = 0; % value of c1 evaluated at first grid point, i.e. rho = 0
    case 'LPV'
        % Parameter dependence
        rho = pgrid('rho',[0,pi/6,pi/3]); % define parameter grid
        c1 = abs(sin(rho)); % define parameter-dependent damping coeff.
end
A = [0, 0, 1, 0;...
     0, 0, 0, 1;...
     -(k1+k2)/m1, k2/m1, -(c1+c2)/m1, c2/m1;...
     k2/m2, -k2/m2, c2/m2, -c2/m2];
B = [zeros(2,3); 1/m1, 1/m1, 0; -1/m2, 0, 0];
C = [-1, 1, 0, 0; 0, -1, 0, 0]; % nonlinear spring f
D = [0, 0, 0; 0, 0, 1];
L = ss(A,B,C,D);
L = set(L,'InputName',{'w(1)';'u(1)';'d(1)'},'OutputName',{'v(1)';'e0(1)'});

% Form the nominal basic plant, i.e. no nonlinearity in the spring
Pbnom = lft(0,L);

% Weight definition: penalize command tracking error at low frequencies
Wo = 0.1*tf(1,[1,0.1])*tf(1,[1,0.01]); % second-order integrator
set(Wo,'InputName',{'e0(1)'},'OutputName',{'e(1)'});

% Weight definition: penalize control effort at high frequencies
Wu = makeweight(1e-2,10,1e2); % high-pass filter, crossover at 10 rad/s
set(Wu,'InputName',{'u(1)'},'OutputName',{'e(2)'});

% Definition of design plant (basic plant plus frequency weights)
H = connect(L,Wo,Wu,{'w';'u';'d'},{'v';'e'});

% Form the nominal design plant, i.e. no nonlinearity in the spring
Hnom = lft(0,H);

% Transpose the plant to be compatible with estimation problem
Ht = transpose(H);
Ht = set(Ht,'InputName',{'w(1)','d(1)','d(2)'},...
    'InputGroup',struct('Uncertain',1,'Disturbance',2:3),...
    'OutputName',{'v(1)','y(1)','z(1)'},...
    'OutputGroup',struct('Uncertain',1,'Measurement',2,'ToBeEstimated',3));

% Definition of uncertainty
Delta = udyn('Delta',[1 1]); % placeholder for odd nonlinearity
set(Delta,'InputName',{'v(1)'},'InputGroup',struct('Uncertain',1),...
    'OutputName',{'w(1)'},'OutputGroup',struct('Uncertain',1),...
    'Notes',{'Delta is the perturbation.'});
Punc = NaN; % uncertain SS object is undefined

% Signal sizes
nw = 1; nv = 1; nu = 1;

%% Definition of IQC multipliers
% MULT = 'zamesfalb';
MULT = 'sector';

switch MULT
    case {'zamesfalb'}
        % Choose parameters for and construct Zames-Falb multiplier
        HM = tf(-0.9,[1 1]);
        % HM = tf(-0.9,[10 1]);
        kM = 1;
        PI = [0, 1+HM; 1+HM', -(2+HM+HM')/kM];
    case {'sector'}
        % Choose parameters for and construct sector bound multiplier
        alpx = -0.9; bet = 1.5;
        PI = [-2*alpx*bet, alpx+bet; alpx+bet, -2];
end

% Construct primal multiplier and j-spectral factorization
[Psi,J] = jspec(PI);
set(Psi,'InputName',{'v(1)','w(1)'},'InputGroup',struct('Uncertain',1:2),...
    'OutputName','vwF','OutputGroup',struct('Uncertainv',1,'Uncertainw',2));

% Construct dual multiplier and j-spectral factorization
DPI = [zeros(nw,nv),-eye(nw);eye(nv),zeros(nv,nw)]...
    *transpose(inv(PI))...
    *[zeros(nv,nw),-eye(nv);eye(nw),zeros(nw,nv)];
% DPI = minreal(DPI);
[DPsi,DJ] = jspec(DPI);
set(DPsi,'InputName',{'v(1)','w(1)'},'InputGroup',struct('Uncertain',1:2),...
    'OutputName','vwF','OutputGroup',struct('Uncertainv',1,'Uncertainw',2));

%% Analysis problem (setting feedforward controller to zero)
% Set PrimalFlag = 1 to use primal problem and = 0 for dual problem
PrimalFlag = 1;

if PrimalFlag
    fprintf('\n ============== ANALYSIS (FF=0, PRIMAL uncertain) ============== \n');
    KFF0 = ss(0); set(KFF0,'InputName','d(1)','OutputName','u(1)');
    G = connect(H,KFF0,{'w','d'},{'v','e'});
    [~,~,~] = wcgain_FF_lmilab_Engine(G,Psi,[],MULT,nw,nv);
else
    fprintf('\n ============== ANALYSIS (FF=0, DUAL uncertain) ============== \n');
    F0 = ss(0); set(F0,'InputName','y(1)','OutputName','zhat(1)');
    sumJnc = ss(sumblk('e(1) = zhat(1) + z(1)'));
    Gt = connect(Ht,F0,sumJnc,{'w','d'},{'v','e'});
    [~,~,~] = wcgain_FF_lmilab_Engine(Gt,DPsi,[],MULT,nw,nv);
end

%% Synthesis problem
fprintf('\n ============== ROBUST SYNTHESIS ============== \n');
[~,Fexp,ubSyn,Mmopt,Plyapopt] = robsyn_FF_lmilab_Engine(Ht,DPsi,[],MULT,nw,nv,nu);
fprintf('\n - [SYN:LMILab] The L2-L2 norm of lft(DELTA,Gt) is copt = %4.2f.\n',ubSyn);

%% LPV re-synthesis
if isa(Ht,'pss') || isa(Ht,'plftss') || isa(Ht,'upss')
    fprintf('\n ============== LPV RE-SYNTHESIS ============== \n');
    [FRE,gmRE,Jpq,INFO,DPSIhat] = robfiltLPVRESYN(Ht,DPsi,Mmopt,ubSyn,nw,nv,nu,'ff');
    fprintf('\n - [LPVRESyn] The L2-L2 norm of lft(DELTA,G) is %4.2f.\n',gmRE);
end

%% A note on the synthesized estimators/feedforwards
% This code provides two different methods to reconstruct the output
% estimator after solving the semidefinite program of the synthesis
% theorem.

% (1) The first method is an explicit procedure that only involves
% algebraic manipulations involving the state-space matrices of H and Psi,
% and the optimal values of X and Z. The variable 'Fexp' that is returned
% by the function 'robsyn_FF_lmilab_Engine' is the explicitly reconstructed
% filter. A note on the explicit reconstruction procedure is given in the
% last paragraph of section 4.1 of the paper. The reconstruction procedure
% closely follows the proof of Lemma 3.1 given in Appendix A of the
% following reference.
%     P. Gahinet and P. Apkarian, ``A linear matrix inequality approach to
%     H-infinity control,'' International Journal of Robust and Nonlinear
%     Control, vol.4, pp.421448, 1994.

% (2) The second method involves solving a second semidefinite program.
% This second semidefinite program is posed as a nominal, gridded LPV
% output feedback controller synthesis problem. Hence, it is solved by
% LPVSYN of LPVTools. Note that the second method is also an explicit
% solution since LPVTools encodes the explicit formulas given in the
% following PhD thesis.
%     F. Wu, ``Control of linear parameter varying systems,'' PhD Thesis,
%     University of California Berkeley, 1995.
% The variable 'FRE' that is returned by the function 'robfiltLPVRESYN' is
% the filter that is returned from the explicit formulas of Wu(1995).

%% Analysis problem (optimal feedforward) (reconstruction using LPVSYN)
fprintf('\n ============== VALIDATION (reconstructed using LPVSYN) ============== \n');
if PrimalFlag
    fprintf('\n ============== ANALYSIS (optimal FF, PRIMAL uncertain) ============== \n');
    KFF = transpose(FRE);
    KFF = set(KFF,'InputName','d(1)','OutputName','u(1)');
    GF = connect(H,KFF,{'w','d'},{'v','e'});
    [~,~,~] = wcgain_FF_lmilab_Engine(GF,Psi,[],MULT,nw,nv);
else
    fprintf('\n ============== ANALYSIS (optimal FF, DUAL uncertain) ============== \n');
    sumJnc = ss(sumblk('e(1) = zhat(1) + z(1)'));
    GtF = connect(Ht,FRE,sumJnc,{'w','d'},{'v','e'});
    [~,~,~] = wcgain_FF_lmilab_Engine(GtF,DPsi,[],MULT,nw,nv);
end

%% Analysis problem (optimal feedforward) (reconstruction using GahinetApkarian)
fprintf('\n ============== VALIDATION (reconstructed using GahinetApkarian) ============== \n');
if PrimalFlag
    fprintf('\n ============== ANALYSIS (optimal FF, PRIMAL uncertain) ============== \n');
    KFFexp = transpose(Fexp);
    KFFexp = set(KFFexp,'InputName','d(1)','OutputName','u(1)');
    GFexp = connect(H,KFFexp,{'w','d'},{'v','e'});
    [~,~,~] = wcgain_FF_lmilab_Engine(GFexp,Psi,[],MULT,nw,nv);
else
    fprintf('\n ============== ANALYSIS (optimal FF, DUAL uncertain) ============== \n');
    sumJnc = ss(sumblk('e(1) = zhat(1) + z(1)'));
    GtFexp = connect(Ht,Fexp,sumJnc,{'w','d'},{'v','e'});
    [~,~,~] = wcgain_FF_lmilab_Engine(GtFexp,DPsi,[],MULT,nw,nv);
end

%% Time-domain simulation of command tracking
fprintf(['\n - If you want to see a simulation of the second mass tracking',...
    '\n\ta step reference command, please run the script tdsim_SMD.m.\n']);
% tdsim_SMD
